//////////////////////////////////////////////////////////////////////
// Simple tromino demo
// -- dovoto
//////////////////////////////////////////////////////////////////////

#include <NDS/NDS.h>

#include <NDS/ARM9/console.h> 
#include <NDS/ARM9/rand.h>

#include <malloc.h>

short *board = 0;
int N = 4;
int slow = 0;

void tile(int n, int holeX, int holeY, int x, int y);

//draw a tile scaled to fill the screen...would be nice if screen height was a power of 2...
void drawTile(int x, int y)
{
	int sizex = 256 / N;
	int xoff = x * sizex;
	
	int sizey;
	int yoff;
	
	int ix, iy;

	//screen height is only 192..so scale it for small N and truncate for large N
	sizey = (N < 128) ? 192 / N : 256/N;

	yoff = y * sizey;

	for(iy = 0; iy < sizey; iy++)
		for(ix = 0; ix < sizex; ix++)
			VRAM_A[ix + xoff + ((iy + yoff) * 256)] = board[x + y * N];

		
}

void wait(void)
{
	int i = 0;

	for(i = 0; i < 30; i++)
	{
		while ((DISP_SR & DISP_IN_VBLANK));
		while (!(DISP_SR & DISP_IN_VBLANK) );

	}
}

void draw(void)
{
	int ix, iy;

	for(ix = 0; ix < N; ix++)
		for(iy = 0; iy < N; iy++)
			drawTile(ix, iy);
}

int main(void)
{
	int holeX = 1, holeY = 1;
	int i;
	int changed = 1;

	//set up the main screen for showing the tiles
	videoSetMode(MODE_FB0); 
	vramSetBankA(VRAM_A_LCD);

	//set up the sub screen
	videoSetModeSub(MODE_0_2D | DISPLAY_BG0_ACTIVE); //sub bg 0 will be used to print text
	vramSetBankC(VRAM_C_SUB_BG); 
	
	SUB_BG0_CR = BG_MAP_BASE(31);
	BG_PALETTE_SUB[255] = RGB15(31,31,31); //set font color
	
	consoleInitDefault((u16*)SCREEN_BASE_BLOCK_SUB(31), (u16*)CHAR_BASE_BLOCK_SUB(0), 16);


	//main loop
	while(1)
	{
		
		scanKeys();

		if(keysDown() & (KEY_B | KEY_X | KEY_Y | KEY_L | KEY_R)) lcdSwap();

		if(keysDown() & KEY_A)
		{
			slow ^= 1;
			changed = 1;
		}
		if(keysDown() & KEY_UP)
		{
			if(N < 256)
			{	
				changed = 1;
				N <<= 1;
			}
		}
		
		if(keysDown() & KEY_DOWN)
		{
			if(N > 2)
			{
				changed  = 1;
				N >>= 1;
			}
		}
		
		if(keysDown() & KEY_TOUCH)
		{
			changed = 1;
			holeX = IPC->touchXpx * N / 256;
			holeY = IPC->touchYpx * N / 192;
 		}
		
		while ((DISP_SR & DISP_IN_VBLANK));
		while (!(DISP_SR & DISP_IN_VBLANK) );

		consoleClear();
		consolePrintSet(0,0);
		consolePrintf("Jason Rogers Tile demo\n");
		consolePrintf("\nPress up to increase N\ndown to decrease N\n");
		consolePrintf("\nPress A to enter slow motion");
		consolePrintf("\nTouch screen to change the hole\n");

		consolePrintf("\n\n2^N = %d", N);
		consolePrintf("\nHole X = %d\nHole Y = %d\n", holeX, holeY);
		consolePrintf("slow = ");

		if(slow) consolePrintf("true");
		else consolePrintf("false");
		
		if(changed)
		{	
			if(board) free(board);
			board = (short*)malloc(N * N * sizeof(short));

			for(i = 0; i < N*N; i++) 
				board[i] = 0;
		
			board[holeX + holeY * N] = 0;

			tile(N, holeX, holeY, 0, 0);
	
			draw();

			changed = 0;
		} 
		

		
	}
	return 0;
}

void tile(int n, int holeX, int holeY, int x, int y)
{
  
	static short tileCount = rand();

	if(n <= 2)
	{
		if(holeX == x)
		{
			board[x + 1 + y * N] = tileCount;
			board[x + 1 + (y + 1) * N] = tileCount;

			if(holeY == y)
				board[x + (y + 1) * N] = tileCount;
			else
         		board[x + y * N] = tileCount;
		}	
		else
		{
			board[x + y * N] = tileCount;
			board[x + (y + 1) * N] = tileCount;
			
			if(holeY == y)
				board[x + 1 + (y + 1) * N] = tileCount;
			else
				board[x + 1 + y * N] = tileCount;
		}
		
	   tileCount = rand() | 1;

	   if(slow) {wait(); draw(); scanKeys(); if(keysDown() & KEY_A) slow ^= 1;}

      return;
	}
	
	if((holeX-x) >= n/2)
	{
		//00
		//0x
		if((holeY-y) >= n/2)
		{
			//draw the center tile
         tile(2, x + n/2, y + n/2, x + n/2 - 1 , y + n/2 - 1);
         
         //fill the remaining four quadrants
         tile(n/2, x + n/2 - 1, y + n/2 -1, x, y);
         tile(n/2, x + n/2, y + n/2 -1, x + n/2, y);
         tile(n/2, holeX, holeY, x + n/2, y + n/2);
         tile(n/2, x + n/2 - 1, y + n/2, x, y + n/2);    
		}
		//0x
		//00
		else
		{
			tile(2, x + n/2, y + n/2 - 1, x + n/2 - 1 , y + n/2 - 1);

         //fill the remaining four quadrants
         tile(n/2, x + n/2 - 1, y + n/2 -1, x, y);
         tile(n/2, holeX, holeY, x + n/2, y);
         tile(n/2, x + n/2, y + n/2, x + n/2, y + n/2);
         tile(n/2, x + n/2 - 1, y + n/2, x, y + n/2);    

		}
	}

	else
	{
		//00
		//x0
		if((holeY-y) >= n/2)
		{
			tile(2, x + n/2 - 1, y  + n/2, x + n/2 - 1 , y + n/2 - 1);

         //fill the remaining four quadrants
         tile(n/2, x + n/2 - 1, y + n/2 -1, x, y);
         tile(n/2, x + n/2, y + n/2 -1, x + n/2, y);
         tile(n/2, x + n/2, y + n/2, x + n/2, y + n/2);
         tile(n/2, holeX, holeY, x, y + n/2);    

		}
		//x0
		//00
		else
		{
			tile(2, x + n/2 - 1, y + n/2 - 1, x + n/2 - 1 , y + n/2 - 1);

         //fill the remaining four quadrants
         tile(n/2, holeX, holeY, x, y);
         tile(n/2, x + n/2, y + n/2 -1, x + n/2, y);
         tile(n/2, x + n/2, y + n/2, x + n/2, y + n/2);
         tile(n/2, x + n/2 - 1, y + n/2, x, y + n/2);    

		}
	}

		
}
